@@ -87,7 +87,6 @@ class Faucet(RyuAppBase):
         }
     _VALVE_SERVICES = {
         EventFaucetMetricUpdate: (None, 5),
-        EventFaucetMaintainStackRoot: (None, valves_manager.STACK_ROOT_STATE_UPDATE_TIME),
         EventFaucetResolveGateways: ('resolve_gateways', 2),
         EventFaucetStateExpire: ('state_expire', 5),
         EventFaucetFastStateExpire: ('fast_state_expire', 2),
@@ -117,6 +116,9 @@ class Faucet(RyuAppBase):
         self.event_sock_hrtbeat_time = int(self.get_setting('EVENT_SOCK_HEARTBEAT') or 0)
         if self.event_sock_hrtbeat_time > 0:
             self._VALVE_SERVICES[EventFaucetEventSockHeartbeat] = ('event_sock_heartbeat', self.event_sock_hrtbeat_time)
+        self.stack_root_state_update_time = int(self.get_setting('STACK_ROOT_STATE_UPDATE_TIME') or 0)
+        if self.stack_root_state_update_time:
+            self._VALVE_SERVICES[EventFaucetMaintainStackRoot] = (None, self.stack_root_state_update_time)
 
     @kill_on_exception(exc_logname)
     def _check_thread_exception(self):
@@ -211,7 +213,7 @@ class Faucet(RyuAppBase):
     @set_ev_cls(EventFaucetMaintainStackRoot, MAIN_DISPATCHER)
     @kill_on_exception(exc_logname)
     def _maintain_stack_root(self, _):
-        self.valves_manager.maintain_stack_root(time.time())
+        self.valves_manager.maintain_stack_root(time.time(), self.stack_root_state_update_time)
 
     @set_ev_cls(EventFaucetEventSockHeartbeat, MAIN_DISPATCHER)
     @kill_on_exception(exc_logname)
